from pydantic import BaseModel
from typing import Any, Optional

class StandardResponse(BaseModel):
    """
    标准API响应格式
    
    该类定义了项目中所有API接口的统一响应格式，包含以下字段：
    - success: 表示请求是否成功处理
    - requestId: 可选请求ID，用于追踪请求
    - errorCode: 可选错误代码，用于标识特定错误类型
    - vaildCode: 验证结果编码(0:成功 1:时间过期 2:并发过高 3:次数耗尽 9:系统异常)
    - errorMsg: 可选错误信息，包含错误详情
    - data: 可选响应数据，包含业务数据
    
    典型使用场景：
    1. 成功响应:
       StandardResponse(success=True, data={"key": "value"})
    
    2. 错误响应:
       StandardResponse(success=False, errorCode=500, errorMsg="Internal error")
    """
    success: bool
    requestId: Optional[str] = None
    errorCode: Optional[int] = None
    #验证结果编码：0：成功；1：时间过期；2：并发过高；3：次数耗尽；9：系统异常
    vaildCode: Optional[int] = None
    errorMsg: Optional[str] = None
    data: Optional[Any] = None
    
    def getSuccess(self):
        return self.success