import re
import sys,os

from src.knowledge.utils.MedicalRecordParser import MedicalRecordParser

current_path = os.getcwd()
sys.path.append(current_path)
import logging
from py_tools.logging import logger
import json
from ..config.site import SiteConfig
from elasticsearch import Elasticsearch, helpers, exceptions
import re
config = SiteConfig()
ELASTICSEARCH_USER = config.get_config("ELASTICSEARCH_USER", "/tmp")
ELASTICSEARCH_PWD = config.get_config("ELASTICSEARCH_PWD", "/tmp")
ELASTICSEARCH_HOST = config.get_config("ELASTICSEARCH_HOST", "/tmp")

class SearchBusiness:
    MIN_SCORE = 5

    def __init__(self):
        self.es = Elasticsearch(hosts=[ELASTICSEARCH_HOST], verify_certs=False, http_auth=(ELASTICSEARCH_USER, ELASTICSEARCH_PWD))
        pass
    def search_nodes_and_edges(self, index, query):
        try:
            response = self.es.search(index=index, body=query)

            hits = response["hits"]["hits"]
            results = []
            for hit in hits:
                source = hit["_source"]
                source["score"] = hit["_score"]
                results.append(source)

            return results
        except exceptions.NotFoundError as e:
            logger.error(f"Index '{index}' not found: {e}")
            return []

    def search_nodes(self, name=None, type=None, id=None, ids=None, limit=10, from_=0):
        try:
            query = {
                "explain": "true",
                "query": {
                    "bool": {
                        "must": [
                            {"term": {"table": "kg_nodes"}}
                        ]
                    }
                },
                "sort": [{"_score": {"order": "desc"}}]
            }

            if name:
                query["query"]["bool"]["must"].append({"match": {"public_kg_nodes_name": name}})
                query["min_score"] = self.MIN_SCORE
            if id:
                query["query"]["bool"]["must"].append({"term": {"public_kg_nodes_id": id}})
            if ids:
                query["query"]["bool"]["must"].append({"terms": {"public_kg_nodes_id": ids}})
                limit = 2000
                from_ = None
            if type:
                query["query"]["bool"]["must"].append({"match": {"public_kg_nodes_category": type}})

            if limit:
                query["size"] = limit
            if from_:
                query["from"] = from_

            results = self.search_nodes_and_edges(index="connector-postgresql-all", query=query)
            return results
        except exceptions.NotFoundError as e:
            logger.error(f"Index not found: {e}")
            return None
        except Exception as e:
            logger.error(f"Search error: {e}")
            return None

    def search_edges(self,name=None,src_id=None,size=None):
        try:
            query = {
                "query": {
                    "bool": {
                        "must": [
                            {"term": {"table": "kg_edges"}}
                        ]
                    }
                },
                "sort": [{"_score": {"order": "desc"}}]
            }
            if name:
                query["query"]["bool"]["must"].append({"match": {"public_kg_edges_category": name}})
                query["min_score"] = self.MIN_SCORE
            if src_id:
                query["query"]["bool"]["must"].append({"term": {"public_kg_edges_src_id": src_id}})
            if size:
                query["size"] = size

            results = self.search_nodes_and_edges(index="connector-postgresql-all", query=query)
            return results
        except exceptions.NotFoundError as e:
            logger.error(f"Index not found: {e}")
            return None
        except Exception as e:
            logger.error(f"Search error: {e}")
            return None

    def _process_symptom_to_diseases(self, symptom_name):
        """
        处理单个症状到疾病的转换逻辑
        :param symptom_name: 症状名称
        :return: 疾病节点列表
        """
        # 存储已处理的疾病ID，避免同义词重复计算
        processed_disease_ids = set()
        simplified_diseases = []
        # 获取症状节点ID
        symptom_nodes = self.search_nodes(name=symptom_name, type="症状", limit=100)
        if not symptom_nodes:
            return simplified_diseases

        for symptom_node in symptom_nodes:
            symptom_id = symptom_node.get("public_kg_nodes_id")
            if not symptom_id:
                continue

            # 查询该症状的'症状同义词'关系获取同义词节点
            # synonym_edges = self.search_edges(name="症状同义词", src_id=symptom_id)
            synonym_ids = {symptom_id}  # 包含原始症状ID
            # if synonym_edges:
            #     for edge in synonym_edges:
            #         synonym_id = edge.get("public_kg_edges_dest_id")
            #         if synonym_id:
            #             synonym_ids.add(synonym_id)

            # 对每个症状ID(包括同义词)查询'常见疾病'关系
            for symptom_id in synonym_ids:
                disease_edges = self.search_edges(name="常见疾病", src_id=symptom_id)
                if not disease_edges:
                    continue

                # 收集所有疾病节点
                for edge in disease_edges:
                    disease_id = edge.get("public_kg_edges_dest_id")
                    if disease_id and disease_id not in processed_disease_ids:
                        processed_disease_ids.add(disease_id)
                        disease_nodes = self.search_nodes(id=disease_id, type="疾病")
                        if disease_nodes:
                            # 只返回需要的字段并重命名
                            for disease in disease_nodes:
                                simplified_diseases.append({
                                    "disease_id": disease.get("public_kg_nodes_id"),
                                    "disease_name": disease.get("public_kg_nodes_name"),
                                    "score": disease.get("score", 0)
                                })

        return simplified_diseases

    def get_symptom_diseases(self, symptom_names):
        """
        根据症状名称列表获取相关疾病列表
        :param symptom_names: 症状名称列表
        :return: 字典，键为症状名称，值为对应的疾病列表
        """
        symptom_disease_map = {}
        disease_dict = {}

        # 对症状名称去重
        unique_symptoms = list(set(symptom_names))
        threshold = len(unique_symptoms) / 2

        for symptom_name in unique_symptoms:
            diseases = self._process_symptom_to_diseases(symptom_name)
            if diseases:
                symptom_disease_map[symptom_name] = diseases
        #json打印symptom_disease_map
        print("symptom_disease_map:",json.dumps(symptom_disease_map, ensure_ascii=False, indent=4))
        #统计每个疾病出现的次数
        for symptom, diseases in symptom_disease_map.items():
            for disease in diseases:
                disease_id = disease.get("disease_id")
                if disease_id in disease_dict:
                    disease_dict[disease_id]["count"] += 1
                else:
                    disease_dict[disease_id] = {
                        "disease_id": disease_id,
                        "disease_name": disease.get("disease_name"),
                        "score": disease.get("score", 0),
                        "count": 1
                    }
        # 按count降序排列并过滤掉count小于阈值的疾病
        sorted_diseases = sorted(disease_dict.values(), key=lambda x: x["count"], reverse=True)
        filtered_diseases = [disease for disease in sorted_diseases if disease["count"] >= threshold]
        #json打印filtered_diseases
        print("filtered_diseases:",json.dumps(filtered_diseases, ensure_ascii=False, indent=4))
        return filtered_diseases

    def get_disease_symptoms(self, disease_name):
        """
        查询疾病的所有症状
        :param disease_name: 疾病名称
        :return: [{"disease": 疾病名称, "symptoms": [症状列表]}]
                  (同名疾病会被合并，症状列表会去重，空症状列表会被过滤)
        """
        try:
            # 1. 根据疾病名称查询疾病列表
            diseases = self.search_nodes(name=disease_name, type="疾病")
            #日志打印diseases的name,id和score
            for disease in diseases:
                logger.info(f"Get disease name: {disease.get('public_kg_nodes_name')}, id: {disease.get('public_kg_nodes_id')}, score: {disease.get('score')}")

            if not diseases:
                return []

            disease_map = {}

            for disease in diseases:
                disease_name = disease.get("public_kg_nodes_name", "")

                # 如果疾病名称已存在，则获取已有记录，否则创建新记录
                if disease_name not in disease_map:
                    disease_map[disease_name] = {
                        "disease": disease_name,
                        "symptoms": []
                    }

                disease_info = disease_map[disease_name]

                # 2. 根据疾病ID查找症状关系
                edges = self.search_edges(name="相关症状", src_id=disease["public_kg_nodes_id"])
                # 增加对'疾病相关症状'关系的查询
                disease_symptom_edges = self.search_edges(name="疾病相关症状", src_id=disease["public_kg_nodes_id"])
                
                # 合并两种关系的查询结果
                if disease_symptom_edges:
                    edges.extend(disease_symptom_edges)
                
                logger.info(f"Get symptoms by disease id: {disease['public_kg_nodes_id']}, result: {edges}")
                if not edges:
                    continue

                # 3. 收集症状节点ID
                symptom_ids = [edge["public_kg_edges_dest_id"] for edge in edges if "public_kg_edges_dest_id" in edge]
                if not symptom_ids:
                    continue

                # 4. 查询症状节点信息
                symptoms = self.search_nodes(ids=symptom_ids)
                if not symptoms:
                    continue

                # 5. 收集症状名称
                for symptom in symptoms:
                    if "public_kg_nodes_name" in symptom:
                        symptom_name = symptom["public_kg_nodes_name"]
                        if symptom_name not in disease_info["symptoms"]:
                            disease_info["symptoms"].append(symptom_name)

            # 过滤掉症状列表为空的疾病
            return [d for d in disease_map.values() if d["symptoms"]]

        except Exception as e:
            logger.error(f"Get disease symptoms error: {e}")
            return []

    def search_concept(self, name, type):
        nodes = self.search_nodes(name=name, type=type)
        concepts = [
            {"name": node["public_kg_nodes_name"], "type": node["public_kg_nodes_category"]}
            for node in nodes
        ] if nodes else []
        return {"concepts": concepts}

    def get_relations(self, name, type):
        # 首先根据name和type查询nodes表数据
        nodes = self.search_nodes(name=name, type=type)
        if not nodes:
            return {}

        result = {}
        for node in nodes:
            node_name = node.get("public_kg_nodes_name", "")
            if not node_name:
                continue

            # 使用node id作为src_id查询edges关系表
            edges = self.search_edges(src_id=node["public_kg_nodes_id"])
            if not edges:
                continue

            # 收集所有目标节点ID
            dest_ids = [edge["public_kg_edges_dest_id"] for edge in edges]
            # 查询目标节点信息
            dest_nodes = self.search_nodes(ids=dest_ids)
            node_map = {n["public_kg_nodes_id"]: n for n in dest_nodes}

            # 构建关系列表
            relations = []
            for edge in edges:
                dest_id = edge["public_kg_edges_dest_id"]
                if dest_id in node_map:
                    relations.append({
                        "relation_type": edge["public_kg_edges_category"],
                        "target_name": node_map[dest_id].get("public_kg_nodes_name", "")
                    })

            # 合并重复node_name的relations
            if node_name in result:
                existing_relations = result[node_name].get("relations", [])
                relations = existing_relations + relations
                
            if relations:
                result[node_name] = {"relations": relations}

        return result

    def get_similar_concepts(self, name, top_k=5):
        # 2. 使用节点名称进行相似搜索
        results = self.search_nodes(name=name, limit=top_k)
        logger.info(f"Similar concepts search results: {results}")
        # 3. 格式化返回结果
        similar_concepts = [
            result["public_kg_nodes_name"]
            for result in results
        ]

        return {"similar_concepts": similar_concepts}

    def get_drug_indications(self, drug_name):
        drugs = self.search_nodes(name=drug_name, type="药品")
        if not drugs:
            return []

        drug_map = {}

        for drug in drugs:
            drug_name = drug.get("public_kg_nodes_name", "")

            if drug_name not in drug_map:
                drug_map[drug_name] = {
                    "drug": drug_name,
                    "indications": []
                }

            drug_info = drug_map[drug_name]

            # 查询药物适应症关系
            edges = self.search_edges(name="适应症", src_id=drug["public_kg_nodes_id"])
            logger.info(f"Get indications by drug id: {drug['public_kg_nodes_id']}, result: {edges}")
            if not edges:
                continue

            # 收集适应症节点ID
            indication_ids = [edge["public_kg_edges_dest_id"] for edge in edges if "public_kg_edges_dest_id" in edge]
            if not indication_ids:
                continue

            # 查询适应症节点信息
            indications = self.search_nodes(ids=indication_ids)
            if not indications:
                continue

            # 收集适应症名称
            for indication in indications:
                if "public_kg_nodes_name" in indication:
                    indication_name = indication["public_kg_nodes_name"]
                    if indication_name not in drug_info["indications"]:
                        drug_info["indications"].append(indication_name)

        # 过滤掉适应症列表为空的药物
        return [d for d in drug_map.values() if d["indications"]]

    def suggest_appropriate_department(self, disease_name: str) -> list:
        """
        根据疾病名称推荐科室
        :param disease_name: 疾病名称
        :return: 返回疾病科室列表，每个元素包含疾病名称和对应的科室列表
        """
        try:
            # 1. 根据疾病名称查询疾病列表
            diseases = self.search_nodes(name=disease_name, type="疾病")
            if not diseases:
                return []

            result = []

            for disease in diseases:
                disease_info = {
                    "disease_name": disease.get("public_kg_nodes_name", ""),
                    "departments": []
                }

                # 2. 查询每个疾病的科室关系
                edges = self.search_edges(name="所属科室", src_id=disease["public_kg_nodes_id"])
                if not edges:
                    continue

                # 3. 收集科室节点ID
                department_ids = [edge["public_kg_edges_dest_id"] for edge in edges
                                if "public_kg_edges_dest_id" in edge]
                if not department_ids:
                    continue

                # 4. 查询科室节点信息
                departments = self.search_nodes(ids=department_ids)
                if not departments:
                    continue

                # 5. 收集科室名称并去重
                department_set = set()
                for dept in departments:
                    if "public_kg_nodes_name" in dept:
                        department_set.add(dept["public_kg_nodes_name"])

                if department_set:
                    disease_info["departments"] = list(department_set)
                    result.append(disease_info)

            return result

        except Exception as e:
            logger.error(f"科室推荐失败: {str(e)}")
            return []


    def _validate_chief_complaint(self, structured_data: dict) -> list:
        """
        验证主诉字段
        :param chief_complaint: 主诉字典
        :return: 错误列表
        """
        chief_complaint = structured_data.get("主诉", {})
        errors = []

        # 检查主要症状是否存在，主要症状是个列表，长度要大于0
        if not chief_complaint.get("主要症状") or not isinstance(chief_complaint["主要症状"], list) or len(chief_complaint["主要症状"]) == 0:
            errors.append("主诉中缺少主要症状或主要症状列表为空")

        # 检查持续时间是否存在
        if not chief_complaint.get("持续时间"):
            errors.append("主诉中缺少持续时间")
        #使用check_time函数检查持续时间是否准确
        if chief_complaint.get("持续时间"):
            is_accurate, error_msg = check_time(chief_complaint["持续时间"])
            if not is_accurate:
                errors.append(f"主诉持续时间'{chief_complaint['持续时间']}'{error_msg}")
        return errors

    def _validate_diagnosis_info(self, structured_data: dict) -> list:
        """
        验证诊断信息
        :param structured_data: 结构化病历数据
        :return: 错误列表
        """
        errors = []
        diagnosis_info = structured_data.get("诊断信息", {})

        if diagnosis_info and diagnosis_info.get("初步诊断"):
            chief_complaint = structured_data.get("主诉", {})
            if chief_complaint and chief_complaint.get("主要症状"):   
                chief_complaint_symptoms = chief_complaint["主要症状"]
 
                related_diseases = self.get_symptom_diseases(chief_complaint_symptoms)
                related_disease_names = [d["disease_name"] for d in related_diseases]
                chief_complaint_symptoms_str = ",".join(chief_complaint_symptoms)
                for diagnosis in diagnosis_info["初步诊断"]:
                    if diagnosis not in related_disease_names:
                        errors.append(f"初步诊断'{diagnosis}'与主诉主要症状'{chief_complaint_symptoms_str}'不相关")

        return errors

    def validate_medical_record(self, medical_record: str, validate_type: str=None) -> dict:
        """
        验证病历数据
        :param medical_record: 病历数据字典
        :param validate_type: 验证类型
        :return: 验证结果字典
        """
        try:
            #将病历数据结构化
            parser = MedicalRecordParser()
            structured_data = parser.parse_medical_record(medical_record)
            #json打印structured_data
            logger.info(f"结构化数据: {json.dumps(structured_data, ensure_ascii=False, indent=4)}")

            # 验证主诉字段格式
            errors = self._validate_chief_complaint(structured_data)

            # 验证诊断信息逻辑
            errors.extend(self._validate_diagnosis_info(structured_data))

            if errors:
                return {"valid": False, "errors": errors}
                
            return {"valid": True, "errors": []}
            
        except Exception as e:
            logger.error(f"病历验证失败: {str(e)}")
            return {"valid": False, "errors": [f"验证过程中发生错误: {str(e)}"]}

    def get_lab_examinations(self, disease_name: str) -> list:
        """
        根据疾病名称获取相关实验室检查项目
        :param disease_name: 疾病名称
        :return: 返回疾病实验室检查列表，每个元素包含疾病名称和对应的实验室检查列表
        """
        try:
            # 1. 根据疾病名称查询疾病列表
            diseases = self.search_nodes(name=disease_name, type="疾病")
            if not diseases:
                return []

            result = []

            for disease in diseases:
                disease_info = {
                    "disease_name": disease.get("public_kg_nodes_name", ""),
                    "lab_examinations": []
                }

                # 2. 查询每个疾病的实验室检查关系
                edges = self.search_edges(name="实验室检查", src_id=disease["public_kg_nodes_id"])
                if not edges:
                    continue

                # 3. 收集实验室检查节点ID
                lab_exam_ids = [edge["public_kg_edges_dest_id"] for edge in edges
                               if "public_kg_edges_dest_id" in edge]
                if not lab_exam_ids:
                    continue

                # 4. 查询实验室检查节点信息
                lab_exams = self.search_nodes(ids=lab_exam_ids)
                if not lab_exams:
                    continue

                # 5. 收集实验室检查名称并去重
                lab_exam_set = set()
                for exam in lab_exams:
                    if "public_kg_nodes_name" in exam:
                        lab_exam_set.add(exam["public_kg_nodes_name"])

                if lab_exam_set:
                    disease_info["lab_examinations"] = list(lab_exam_set)
                    result.append(disease_info)

            return result

        except Exception as e:
            logger.error(f"实验室检查查询失败: {str(e)}")
            return []

    def get_auxiliary_examinations(self, disease_name: str) -> list:
        """
        根据疾病名称获取相关辅助检查项目
        :param disease_name: 疾病名称
        :return: 返回疾病辅助检查列表，每个元素包含疾病名称和对应的辅助检查列表
        """
        try:
            # 1. 根据疾病名称查询疾病列表
            diseases = self.search_nodes(name=disease_name, type="疾病")
            if not diseases:
                return []

            result = []

            for disease in diseases:
                disease_info = {
                    "disease_name": disease.get("public_kg_nodes_name", ""),
                    "auxiliary_examinations": []
                }

                # 2. 查询每个疾病的辅助检查关系
                edges = self.search_edges(name="辅助检查", src_id=disease["public_kg_nodes_id"])
                if not edges:
                    continue

                # 3. 收集辅助检查节点ID
                aux_exam_ids = [edge["public_kg_edges_dest_id"] for edge in edges
                               if "public_kg_edges_dest_id" in edge]
                if not aux_exam_ids:
                    continue

                # 4. 查询辅助检查节点信息
                aux_exams = self.search_nodes(ids=aux_exam_ids)
                if not aux_exams:
                    continue

                # 5. 收集辅助检查名称并去重
                aux_exam_set = set()
                for exam in aux_exams:
                    if "public_kg_nodes_name" in exam:
                        aux_exam_set.add(exam["public_kg_nodes_name"])

                if aux_exam_set:
                    disease_info["auxiliary_examinations"] = list(aux_exam_set)
                    result.append(disease_info)

            return result

        except Exception as e:
            logger.error(f"辅助检查查询失败: {str(e)}")
            return []



def check_time(time_str: str) :
    """
    检查主诉中的时间描述是否准确
    :param time_str: 主诉文本
    :return: (是否合理, 错误原因)
    """
    # 定义可接受的时间单位
    time_units = ["小时", "天", "日", "周", "月", "年"]

    # 正则匹配时间描述（如"1天"、"2 小时"、"约3天"）
    time_pattern = re.compile(
        r"(\d+)\s*([小时天日周月年]+)|(约|大约|近|接近)?\s*(\d+)\s*([小时天日周月年]+)"
    )

    match = time_pattern.search(time_str)
    if not match:
        return (False, "未找到明确的时间描述")

    # 提取数值和单位
    num = int(match.group(1)) if match.group(1) else int(match.group(4))
    unit = match.group(2) if match.group(2) else match.group(5)

    # 检查单位是否合法
    if unit not in time_units:
        return (False, f"时间单位'{unit}'不规范，应为{time_units}")

    # 检查数值是否合理
    if num <= 0:
        return (False, f"时间数值'{num}'不能为0或负数")
    if unit in ["小时"] and num > 720:  # 假设超过30天（720小时）不合理
        return (False, f"时间数值'{num}{unit}'可能过大")
    if unit in ["天", "日"] and num > 365:  # 超过1年应改用"年"
        return (False, f"时间数值'{num}{unit}'建议改用年描述")

    return (True, f"时间描述'{num}{unit}'合理")

if __name__ == "__main__":
    search_biz = SearchBusiness()
    index=""
    query=""
    results = search_biz.search_nodes_and_edges(index=index, query=query)
    print(results)